# -*- encoding: utf-8 -*-
require 'spec_helper'
#コミック

describe Comic do
  before do
    @admin = FactoryGirl.create :admin
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @user = FactoryGirl.create( :user_yas)
    @author = FactoryGirl.create :author, :user_id => @user.id
    @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
    @other_user = FactoryGirl.create( :user_yas)
    @other_author = FactoryGirl.create :author, :user_id => @other_user.id
    @other_artist = FactoryGirl.create :artist_yas, :author_id => @other_author.id
  end
  
  describe '検証に於いて' do
    before do
      @comic = FactoryGirl.build :comic, :author_id => @author.id
    end
    
    context 'オーソドックスなデータのとき' do
      it '下限データが通る' do
        @comic.title = 'a'
        @comic.visible = 0
        @comic.should be_valid
      end
      it '上限データが通る' do
        @comic.title = 'a'*100
        @comic.visible = 1
        @comic.should be_valid
      end
    end
    
    context 'titleを検証するとき' do
      it 'nullなら失敗する' do
        @comic.title = nil
        @comic.should_not be_valid
      end
      it '100文字以上なら失敗する' do
        @comic.title = 'a'*101
        @comic.should_not be_valid
      end
    end
    context 'visibleを検証するとき' do
      it 'nullなら失敗する' do
        @comic.visible = nil
        @comic.should_not be_valid
      end
      it '負なら失敗する' do
        @comic.visible = -1
        @comic.should_not be_valid
      end
      it '2以上なら失敗する' do
        @comic.visible = 2
        @comic.should_not be_valid
      end
    end
  end
  
  describe 'デフォルト値補充に於いて' do
    it 'visibleが0になっている' do
      @comic = FactoryGirl.build :comic, :visible => nil
      @comic.supply_default
      @comic.visible.should eq 0
    end
  end
  
  describe '上書き補充に於いて' do
    it '作家idが設定されている' do
      @comic = FactoryGirl.build :comic, :author_id => nil
      @comic.overwrite @author
      @comic.author_id.should eq @author.id
    end
  end
  
  describe '所持判定に於いて' do
    before do
      @comic = FactoryGirl.build :comic, :author_id => @author.id
    end
    it '自分のコミックならyes' do
      @comic.own?(@author).should == true
    end
    it '他人のコミックならno' do
      @comic.own?(@other_author).should == false
    end
    it 'パラメータが作家でないならno' do
      @comic.own?(nil).should == false
    end
  end
  
  describe '閲覧許可に於いて' do
    before do
      @comic = FactoryGirl.build :comic, :author_id => @author.id
    end
    context '検査対象がnil(ゲスト)のとき' do
      context 'クローズドモードのとき' do
        before do
          MagicNumber['run_mode'] = 1
        end
        it '不許可を返す。' do
          r = @comic.visible?(nil)
          r.should be_false
        end
      end
      context 'オープンモードのとき' do
        before do
          MagicNumber['run_mode'] = 0
        end
        it '公開コミックなら許可する' do
          Comic.any_instance.stub(:visible).and_return(1)
          r = @comic.visible?(nil)
          r.should be_true
        end
        it '非公開コミックなら許可しない' do
          Comic.any_instance.stub(:visible).and_return(0)
          r = @comic.visible?(nil)
          r.should be_false
        end
      end
    end
    context '検査対象が作家のとき' do
      it '自分のコミックなら許可する' do
        Comic.any_instance.stub(:own?).and_return(true)
        Comic.any_instance.stub(:visible).and_return(0)
        r = @comic.visible?(@author)
        r.should == true
      end
      it '他人の非公開コミックなら許可しない' do
        Comic.any_instance.stub(:own?).and_return(false)
        Comic.any_instance.stub(:visible).and_return(0)
        r = @comic.visible?(@author)
        r.should == false
      end
      it '他人のコミックでも公開なら許可する' do
        Comic.any_instance.stub(:own?).and_return(false)
        Comic.any_instance.stub(:visible).and_return(1)
        r = @comic.visible?(@author)
        r.should == true
      end
    end
    context '検査対象がそれ以外のとき' do
      it '不許可を返す。' do
        r = @comic.visible?(@admin)
        r.should be_false
      end
    end
  end
  
  describe '一覧取得に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        Comic.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        Comic.page().should eq 1
      end
      it '0以下の場合は1になる' do
        Comic.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        Comic.page_size('7').should eq 7
      end
      it 'nilの場合はComic.default_page_sizeになる' do
        Comic.page_size().should eq Comic.default_page_size
      end
      it '0以下の場合はComic.default_page_sizeになる' do
        Comic.page_size('0').should eq Comic.default_page_size
      end
      it 'Comic.max_page_sizeを超えた場合はComic.max_page_sizeになる' do
        Comic.page_size('1000').should eq Comic.max_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        Comic.stub(:list_opt).with(any_args).and_return({})
        Comic.should_receive(:list_opt).with(any_args).exactly(1)
        r = Comic.list
      end
    end
    it 'リストを返す' do
      c = Comic.list
      c.should eq [@comic]
    end
    it '非公開コミックは(自分のコミックであっても)含んでいない' do
      FactoryGirl.create :comic, :author_id => @author.id, :visible => 0
      c = Comic.list
      c.should eq [@comic]
    end
    it '時系列で並んでいる' do
      #公開コミックは(他人のコミックであっても)含んでいる
      v = FactoryGirl.create :comic, :author_id => @other_author.id, :updated_at => Time.now + 100
      c = Comic.list
      c.should eq [v, @comic]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @comic2 = FactoryGirl.create :comic, :author_id => @author.id, :updated_at => Time.now + 100
        @comic3 = FactoryGirl.create :comic, :author_id => @author.id, :updated_at => Time.now + 200
        @comic4 = FactoryGirl.create :comic, :author_id => @author.id, :updated_at => Time.now + 300
        @comic5 = FactoryGirl.create :comic, :author_id => @author.id, :updated_at => Time.now + 400
        Comic.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        c = Comic.list
        c.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        c = Comic.list(1)
        c.should eq [@comic5, @comic4]
      end
      it 'page=2なら中間2件を返す' do
        c = Comic.list(2)
        c.should eq [@comic3, @comic2]
      end
      it 'page=3なら先頭1件を返す' do
        c = Comic.list(3)
        c.should eq [@comic]
      end
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @comic2 = FactoryGirl.create :comic, :author_id => @author.id, :updated_at => Time.now + 100
        @comic3 = FactoryGirl.create :comic, :author_id => @author.id, :updated_at => Time.now + 200
        @comic4 = FactoryGirl.create :comic, :author_id => @author.id, :updated_at => Time.now + 300
        @comic5 = FactoryGirl.create :comic, :author_id => @author.id, :updated_at => Time.now + 400
        Comic.stub(:default_page_size).and_return(2)
      end
      it '件数0は全件(5件)を返す' do
        r = Comic.list 5, 0
        r.should have(5).items 
      end
    end
  end
  describe '一覧取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = Comic.list_opt
      r.has_key?(:include).should be_true
    end
    it '2つの項目を含んでいる' do
      r = Comic.list_opt[:include]
      r.should have(2).items
    end
    it 'ストーリーを含んでいる' do
      r = Comic.list_opt[:include]
      r.has_key?(:stories).should be_true
    end
      it 'ストーリーはコマを含んでいる' do
        r = Comic.list_opt[:include]
        r[:stories].has_key?(:panel).should be_true
      end
    it '作家を含んでいる' do
      r = Comic.list_opt[:include]
      r.has_key?(:author).should be_true
    end
  end
  describe 'json一覧出力オプションに於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id, :picture_id => @p.id
      @sbt = FactoryGirl.create :speech_balloon_template
      @comic = FactoryGirl.create :comic, :author_id => @author.id, :visible => 1
      @panel = FactoryGirl.create :panel, :author_id => @author.id, :publish => 1
      @story = FactoryGirl.create :story, :author_id => @author.id, :comic_id => @comic.id, :panel_id => @panel.id
    end
    it 'ストーリーを含んでいる' do
      r = Comic.list.to_json Comic.list_json_opt
      j = JSON.parse r
      i = j.first
      i.has_key?('stories').should be_true
    end
      it 'ストーリーはコマを含んでいる' do
        r = Comic.list.to_json Comic.list_json_opt
        j = JSON.parse r
        i = j.first
        s = i['stories'].first
        s.has_key?('panel').should be_true
      end
    it '作家を含んでいる' do
      r = Comic.list.to_json Comic.list_json_opt
      j = JSON.parse r
      i = j.first
      i.has_key?('author').should be_true
    end
  end
  
  describe '自分のコミック一覧取得に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        Comic.stub(:list_opt).with(any_args).and_return({})
        Comic.should_receive(:list_opt).with(any_args).exactly(1)
        r = Comic.mylist @author
      end
    end
    it 'リストを返す' do
      c = Comic.mylist @author
      c.should eq [@comic]
    end
    it '時系列で並んでいる' do
      nc = FactoryGirl.create :comic, :author_id => @author.id, :updated_at => Time.now + 100
      cl = Comic.mylist @author
      cl.should eq [nc, @comic]
    end
    it '他人のコミックは公開でも含まない' do
      nc = FactoryGirl.create :comic, :author_id => @other_author.id, :visible => 1
      cl = Comic.mylist @author
      cl.should eq [@comic]
    end
    it '自分のコミックは非公開でも含んでいる' do
      nc = FactoryGirl.create :comic, :author_id => @author.id, :visible => 0, :updated_at => Time.now + 100
      cl = Comic.mylist @author
      cl.should eq [nc, @comic]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @comic2 = FactoryGirl.create :comic, :author_id => @author.id, :updated_at => Time.now + 100
        @comic3 = FactoryGirl.create :comic, :author_id => @author.id, :updated_at => Time.now + 200
        @comic4 = FactoryGirl.create :comic, :author_id => @author.id, :updated_at => Time.now + 300
        @comic5 = FactoryGirl.create :comic, :author_id => @author.id, :updated_at => Time.now + 400
      end
      it '通常は2件を返す' do
        c = Comic.mylist @author, 1, 2
        c.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        c = Comic.mylist(@author, 1, 2)
        c.should eq [@comic5, @comic4]
      end
      it 'page=2なら中間2件を返す' do
        c = Comic.mylist(@author, 2, 2)
        c.should eq [@comic3, @comic2]
      end
      it 'page=3なら先頭1件を返す' do
        c = Comic.mylist(@author, 3, 2)
        c.should eq [@comic]
      end
    end
    context 'DBに5件あって1ページの件数を0件に変えたとして' do
      before do
        @comic2 = FactoryGirl.create :comic, :author_id => @author.id, :updated_at => Time.now + 100
        @comic3 = FactoryGirl.create :comic, :author_id => @author.id, :updated_at => Time.now + 200
        @comic4 = FactoryGirl.create :comic, :author_id => @author.id, :updated_at => Time.now + 300
        @comic5 = FactoryGirl.create :comic, :author_id => @author.id, :updated_at => Time.now + 400
        Author.stub(:default_comic_page_size).and_return(2)
      end
      it '通常は全件(5件)を返す' do
        r = Comic.mylist @author, 5, 0
        r.should have(5).items 
      end
    end
  end
  
  describe '単体取得に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        Comic.stub(:show_opt).with(any_args).and_return({})
        Comic.should_receive(:show_opt).with(any_args).exactly(1)
        r = Comic.show @comic.id, @author
      end
      it '閲覧許可を問い合わせている' do
        Comic.any_instance.stub(:visible?).with(any_args).and_return(true)
        Comic.any_instance.should_receive(:visible?).with(any_args).exactly(1)
        r = Comic.show @comic.id, @author
      end
    end
    it '指定のコミックを返す' do
      c = Comic.show @comic.id, @author
      c.should eq @comic
    end
    context '閲覧許可が出なかったとき' do
      it '403Forbidden例外を返す' do
        Comic.any_instance.stub(:visible?).and_return(false)
        lambda{
          Comic.show @comic.id, @author
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しないコミックを開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          Comic.show 110, @author
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  
  describe '編集取得に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        Comic.stub(:show_opt).with(any_args).and_return({})
        Comic.should_receive(:show_opt).with(any_args).exactly(1)
        r = Comic.edit @comic.id, @author
      end
      it '所持判定を問い合わせている' do
        Comic.any_instance.stub(:own?).with(any_args).and_return(true)
        Comic.any_instance.should_receive(:own?).with(any_args).exactly(1)
        r = Comic.edit @comic.id, @author
      end
    end
    it '指定のコミックを返す' do
      Comic.any_instance.stub(:own?).and_return(true)
      c = Comic.edit @comic.id, @author.id
      c.should eq @comic
    end
    context '他人のコミックを開こうとしたとき' do
      it '403Forbidden例外を返す' do
        Comic.any_instance.stub(:own?).and_return(false)
        lambda{
          Comic.edit @comic.id, @author
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しないコミックを開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          Comic.edit 110, @author
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  describe '単体取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = Comic.show_opt
      r.has_key?(:include).should be_true
    end
    it '2つの項目を含んでいる' do
      r = Comic.show_opt[:include]
      r.should have(2).items
    end
    it '作家を含んでいる' do
      r = Comic.show_opt[:include]
      r.has_key?(:author).should be_true
    end
    it 'ストーリーを含んでいる' do
      r = Comic.show_opt[:include]
      r.has_key?(:stories).should be_true
    end
      it 'ストーリーはコマを含んでいる' do
        r = Comic.show_opt[:include]
        r[:stories].has_key?(:panel).should be_true
      end
  end
  describe 'json単体出力オプションに於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id, :picture_id => @p.id
      @sbt = FactoryGirl.create :speech_balloon_template
      @comic = FactoryGirl.create :comic, :author_id => @author.id, :visible => 1
      @panel = FactoryGirl.create :panel, :author_id => @author.id, :publish => 1
      @story = FactoryGirl.create :story, :author_id => @author.id, :comic_id => @comic.id, :panel_id => @panel.id
    end
    it 'ストーリーを含んでいる' do
      r = Comic.show(@comic.id, @author).to_json Comic.show_json_opt
      j = JSON.parse r
      i = j
      i.has_key?('stories').should be_true
    end
      it 'ストーリーはコマを含んでいる' do
        r = Comic.show(@comic.id, @author).to_json Comic.show_json_opt
        j = JSON.parse r
        i = j
        s = i['stories'].first
        s.has_key?('panel').should be_true
      end
    it '作家を含んでいる' do
      r = Comic.show(@comic.id, @author).to_json Comic.show_json_opt
      j = JSON.parse r
      i = j
      i.has_key?('author').should be_true
    end
  end
  
  describe '削除に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id, :publish => 1
      @story = FactoryGirl.create :story, :author_id => @author.id, :comic_id => @comic.id, :panel_id => @panel.id
      @other_comic = FactoryGirl.create :comic, :author_id => @author.id
      @other_story = FactoryGirl.create :story, :author_id => @author.id, :comic_id => @other_comic.id, :panel_id => @panel.id
    end
    context 'つつがなく終わるとき' do
      it '自身を削除する' do
        lambda {
          r = @comic.destroy_with_story
        }.should change(Comic, :count).by(-1)
        lambda {
          r = Comic.find @comic.id
        }.should raise_error
      end
      it '自身にリンクしているストーリーをすべて削除する' do
        lambda {
          r = @comic.destroy_with_story
        }.should change(Story, :count).by(-1)
        lambda {
          r = Story.find @story.id
        }.should raise_error
      end
      it 'Trueを返す' do
        r = @comic.destroy_with_story
        r.should be_true
      end
    end
    context '削除に失敗したとき' do
      before do
        Story.any_instance.stub(:destroy).with(any_args).and_return(false)
      end
      it 'Falseを返す' do
        r = @comic.destroy_with_story
        r.should be_false
      end
      it 'ロールバックしている' do
        lambda {
          r = @comic.destroy_with_story
        }.should_not change(Comic, :count)
        lambda {
          r = @comic.destroy_with_story
        }.should_not change(Story, :count)
      end
    end
  end
end
